from django.conf import settings


PDFKIT_OPTIONS = {
    'page-size': 'Letter',
    'margin-top': '1.2in',
    'margin-right': '0.75in',
    'margin-bottom': '1.2in',
    'margin-left': '0.75in',
    'encoding': "UTF-8",
    'no-outline': None,
    'quiet': '',
}

DEFAULT_LOGGING_FORMATTER = {
    'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
}

DEFAULT_LOGGING_HANDLER = {
    'level': 'INFO',
    'class': 'logging.StreamHandler',
    'formatter': 'lightpdf'
}

DEFAULT_LOGGER = {
    'handlers': ['lightpdf'],
    'level': 'INFO',
}


LOGGING_VERSION = settings.LOGGING.get('version', 1)
LOGGING_FORMATTER = settings.LOGGING.get('formatters', {}).get('lightpdf', DEFAULT_LOGGING_FORMATTER)
LOGGING_HANDLER = settings.LOGGING.get('handlers', {}).get('lightpdf', DEFAULT_LOGGING_HANDLER)
LOGGING_LOGGER = settings.LOGGING.get('logger', {}).get('lightpdf', DEFAULT_LOGGER)

LOGGING_CONFIG = {
    'version': LOGGING_VERSION,
    'disable_existing_loggers': False,
    'formatters': {
        'lightpdf': LOGGING_FORMATTER,
    },
    'handlers': {
        'lightpdf': LOGGING_HANDLER,
    },
    'loggers': {
        'lightpdf': LOGGING_LOGGER,
    }
}


LIGHTPDF_PDFKIT_OPTIONS = getattr(settings, 'LIGHTPDF_PDFKIT_OPTIONS', PDFKIT_OPTIONS)
LIGHTPDF_LOGGING_CONFIG = getattr(settings, 'LIGHTPDF_LOGGING_CONFIG', LOGGING_CONFIG)



import logging

import pdfkit

from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse
from django.template.loader import render_to_string

from lightpdf.settings import LIGHTPDF_PDFKIT_OPTIONS, LIGHTPDF_LOGGING_CONFIG


logging.config.dictConfig(LIGHTPDF_LOGGING_CONFIG)
logger = logging.getLogger(__name__)


class PdfGeneratorMixin(object):
    pdf_name = None

    def get_pdf_name(self):
        if self.pdf_name is None:
            raise ImproperlyConfigured(
                "PdfGeneratorMixin requires either a definition of "
                "'pdf_name' or an implementation of 'get_pdf_name()'")

    def render_to_string(self):
        return render_to_string(self.template_name, self.get_context_data())

    def render_to_pdf(self):
        return pdfkit.from_string(self.render_to_string(), False, options=LIGHTPDF_PDFKIT_OPTIONS)

    def render_to_pdf_response(self):
        try:
            pdf_name = self.get_pdf_name()
            pdf = self.render_to_pdf()
            response = HttpResponse(pdf, content_type='application/pdf')
            response['Content-Disposition'] = 'filename="%s.pdf"'.format(pdf_name)
        except Exception as e:
            logger.error(e)
            response = HttpResponse('We had some errors while generating PDF')
        return response



from django.test import RequestFactory, TestCase
from django.views.generic import TemplateView

from lightpdf.mixins import PdfGeneratorMixin


class PdfView(PdfGeneratorMixin, TemplateView):
    template_name = 'blah.html'

    def get(self, request, *args, **kwargs):
        return self.render_to_pdf_response()

    def render_to_string(self):
        return '<p>blah</p>'


class PdfGeneratorMixinTestCase(TestCase):

    factory = RequestFactory()

    def tearDown(self):
        PdfView.pdf_name = None

    def test_pdf_name_not_specified(self):
        request = self.factory.get('/rand')
        view = PdfView.as_view()
        response = view(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, 'We had some errors while generating PDF')
        self.assertEqual(response['content-type'], 'text/html; charset=utf-8')

    def test_pdf_ok(self):
        request = self.factory.get('/rand')
        PdfView.pdf_name = 'blah'
        view = PdfView.as_view()
        response = view(request)
        self.assertEqual(response['content-type'], 'application/pdf')
